@@ -67,8 +67,9 @@ typedef struct {
 
 typedef struct {
 	EvFileExporterFormat format;
-	PopplerPSFile *ps_file;
 
+	gboolean landscape;
+	
 	/* Pages per sheet */
 	gint pages_per_sheet;
 	gint pages_printed;
@@ -79,6 +80,8 @@ typedef struct {
 	
 #ifdef HAVE_CAIRO_PRINT
 	cairo_t *cr;
+#else
+	PopplerPSFile *ps_file;
 #endif
 } PdfPrintContext;
 
@@ -126,6 +129,7 @@ static EvLink     *ev_link_from_action      (PdfDocument       *pdf_document,
 static void        pdf_document_search_free (PdfDocumentSearch *search);
 static void        pdf_print_context_free   (PdfPrintContext   *ctx);
 
+#ifdef HAVE_FORMS
 G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
                          {
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT,
@@ -138,10 +142,8 @@ G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
 							pdf_document_document_links_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_IMAGES,
 							pdf_document_document_images_iface_init);
-#ifdef HAVE_FORMS
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FORMS,
 							pdf_document_document_forms_iface_init);
-#endif /* HAVE_FORMS */
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FONTS,
 							pdf_document_document_fonts_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND,
@@ -153,7 +155,31 @@ G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_TRANSITION,
 							pdf_document_page_transition_iface_init);
 			 });
-
+#else /* !HAVE_FORMS */
+G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
+                         {
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT,
+							pdf_document_document_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_SECURITY,
+							pdf_document_security_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_THUMBNAILS,
+							pdf_document_document_thumbnails_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_LINKS,
+							pdf_document_document_links_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_IMAGES,
+							pdf_document_document_images_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FONTS,
+							pdf_document_document_fonts_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND,
+							pdf_document_find_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_FILE_EXPORTER,
+							pdf_document_file_exporter_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_SELECTION,
+							pdf_selection_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_TRANSITION,
+							pdf_document_page_transition_iface_init);
+			 });
+#endif /* HAVE_FORMS */
 
 static void
 set_rc_data (PdfDocument     *pdf_document,
@@ -477,7 +503,7 @@ pdf_document_render (EvDocument      *document,
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
 	
-	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+	surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
 					      width, height);
 	memset (cairo_image_surface_get_data (surface), 0xff,
 		cairo_image_surface_get_height (surface) *
@@ -1270,7 +1296,15 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	pixbuf = poppler_page_get_thumbnail (poppler_page);
-	if (!pixbuf) {
+	if (pixbuf) {
+		/* Rotate provided thumbnail if needed */
+		GdkPixbuf *rotated_pixbuf;
+
+		rotated_pixbuf = gdk_pixbuf_rotate_simple (pixbuf,
+							   (GdkPixbufRotation) (360 - rc->rotation));
+		g_object_unref (pixbuf);
+		pixbuf = rotated_pixbuf;
+	} else {
 		/* There is no provided thumbnail.  We need to make one. */
 		pixbuf = make_thumbnail_for_page (pdf_document, poppler_page, rc);
 	}
@@ -1528,15 +1562,16 @@ pdf_print_context_free (PdfPrintContext *ctx)
 	if (!ctx)
 		return;
 
-	if (ctx->ps_file) {
-		poppler_ps_file_free (ctx->ps_file);
-		ctx->ps_file = NULL;
-	}
 #ifdef HAVE_CAIRO_PRINT
 	if (ctx->cr) {
 		cairo_destroy (ctx->cr);
 		ctx->cr = NULL;
 	}
+#else
+	if (ctx->ps_file) {
+		poppler_ps_file_free (ctx->ps_file);
+		ctx->ps_file = NULL;
+	}
 #endif
 	g_free (ctx);
 }
@@ -1548,7 +1583,7 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	PdfPrintContext *ctx;
 	gdouble width, height;
-	gboolean change_orient = FALSE;
+	gboolean landscape, change_orient = FALSE;
 #ifdef HAVE_CAIRO_PRINT
 	cairo_surface_t *surface = NULL;
 #endif
@@ -1560,6 +1595,9 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 	ctx->format = fc->format;
 	ctx->pages_per_sheet = fc->pages_per_sheet;
 
+	landscape = (fc->orientation == EV_FILE_EXPORTER_LANDSCAPE);
+	change_orient = landscape;
+	
 	switch (fc->pages_per_sheet) {
 	        default:
 	        case 1:
@@ -1567,18 +1605,18 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			ctx->pages_y = 1;
 			break;
 	        case 2:
-			change_orient = TRUE;
-			ctx->pages_x = 2;
-			ctx->pages_y = 1;
+			landscape = !landscape;
+			ctx->pages_x = 1;
+			ctx->pages_y = 2;
 			break;
 	        case 4:
 			ctx->pages_x = 2;
 			ctx->pages_y = 2;
 			break;
 	        case 6:
-			change_orient = TRUE;
-			ctx->pages_x = 3;
-			ctx->pages_y = 2;
+			landscape = !landscape;
+			ctx->pages_x = 2;
+			ctx->pages_y = 3;
 			break;
 	        case 9:
 			ctx->pages_x = 3;
@@ -1590,6 +1628,8 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			break;
 	}
 
+	ctx->landscape = landscape;
+
 	if (change_orient) {
 		width = fc->paper_height;
 		height = fc->paper_width;
@@ -1598,8 +1638,19 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 		height = fc->paper_height;
 	}
 
-	ctx->page_width = width / ctx->pages_x;
-	ctx->page_height = height / ctx->pages_y;
+	if (landscape) {
+		gint tmp;
+
+		tmp = ctx->pages_x;
+		ctx->pages_x = ctx->pages_y;
+		ctx->pages_y = tmp;
+
+		ctx->page_width = height / ctx->pages_x;
+		ctx->page_height = width / ctx->pages_y;
+	} else {
+		ctx->page_width = width / ctx->pages_x;
+		ctx->page_height = height / ctx->pages_y;
+	}
 
 	ctx->pages_printed = 0;
 
@@ -1626,6 +1677,16 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 
 #ifdef HAVE_CAIRO_PRINT
 	ctx->cr = cairo_create (surface);
+	if (landscape) {
+		cairo_matrix_t matrix;
+		
+		cairo_translate (ctx->cr, width, 0);
+		cairo_matrix_init (&matrix,
+				   0,  1,
+				   -1,  0,
+				   0,  0);
+		cairo_transform (ctx->cr, &matrix);
+	}
 	cairo_surface_destroy (surface);
 #endif
 }
@@ -1655,9 +1716,15 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	cairo_translate (ctx->cr,
 			 x * ctx->page_width,
 			 y * ctx->page_height);
-	cairo_scale (ctx->cr,
-		     ctx->page_width / page_width,
-		     ctx->page_height / page_height);
+	if (ctx->landscape) {
+		cairo_scale (ctx->cr,
+			     ctx->page_height / page_height,
+			     ctx->page_height / page_height);
+	} else {
+		cairo_scale (ctx->cr,
+			     ctx->page_width / page_width,
+			     ctx->page_height / page_height);
+	}
 
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	poppler_page_render (poppler_page, ctx->cr);
